# Copyright 2017 Brocade Communications Systems, Inc
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"Misc constants"

POW_2_7  = round( pow( 2,  7 ))
POW_2_8  = round( pow( 2,  8 ))
POW_2_15 = round( pow( 2, 15 ))
POW_2_16 = round( pow( 2, 16 ))
POW_2_31 = round( pow( 2, 31 ))
POW_2_32 = round( pow( 2, 32 ))
POW_2_63 = round( pow( 2, 63 ))
POW_2_64 = round( pow( 2, 64 ))
